import { Injectable, NotFoundException, Inject } from '@nestjs/common';
import { eq } from 'drizzle-orm';
import { DRIZZLE } from '../../../database/database.module';
import * as schema from '../../../database/schema';
import { CreateGroupDto } from '../dto/create-group.dto';
import { UpdateGroupDto } from '../dto/update-group.dto';

@Injectable()
export class GroupsService {
  constructor(@Inject(DRIZZLE) private readonly db: any) {}

  /**
   * Create a new group
   */
  async create(createGroupDto: CreateGroupDto) {
    const [group] = await this.db
      .insert(schema.groups)
      .values({
        ...createGroupDto,
      })
      .returning();
    return group;
  }

  /**
   * Find all groups
   */
  async findAll() {
    return this.db.select().from(schema.groups);
  }

  /**
   * Find groups by project ID
   */
  async findByProjectId(projectId: string) {
    return this.db
      .select()
      .from(schema.groups)
      .where(eq(schema.groups.projectId, projectId));
  }

  /**
   * Find a group by ID
   */
  async findById(id: string) {
    const [group] = await this.db
      .select()
      .from(schema.groups)
      .where(eq(schema.groups.id, id));
    
    if (!group) {
      throw new NotFoundException(`Group with ID ${id} not found`);
    }
    
    return group;
  }

  /**
   * Update a group
   */
  async update(id: string, updateGroupDto: UpdateGroupDto) {
    const [group] = await this.db
      .update(schema.groups)
      .set({
        ...updateGroupDto,
        updatedAt: new Date(),
      })
      .where(eq(schema.groups.id, id))
      .returning();
    
    if (!group) {
      throw new NotFoundException(`Group with ID ${id} not found`);
    }
    
    return group;
  }

  /**
   * Delete a group
   */
  async remove(id: string) {
    const [group] = await this.db
      .delete(schema.groups)
      .where(eq(schema.groups.id, id))
      .returning();
    
    if (!group) {
      throw new NotFoundException(`Group with ID ${id} not found`);
    }
    
    return group;
  }

  /**
   * Add a person to a group
   */
  async addPersonToGroup(groupId: string, personId: string) {
    // Check if the group exists
    await this.findById(groupId);
    
    // Check if the person exists
    const [person] = await this.db
      .select()
      .from(schema.persons)
      .where(eq(schema.persons.id, personId));
    
    if (!person) {
      throw new NotFoundException(`Person with ID ${personId} not found`);
    }
    
    // Check if the person is already in the group
    const [existingRelation] = await this.db
      .select()
      .from(schema.personToGroup)
      .where(eq(schema.personToGroup.personId, personId))
      .where(eq(schema.personToGroup.groupId, groupId));
    
    if (existingRelation) {
      return existingRelation;
    }
    
    // Add the person to the group
    const [relation] = await this.db
      .insert(schema.personToGroup)
      .values({
        personId,
        groupId,
      })
      .returning();
    
    return relation;
  }

  /**
   * Remove a person from a group
   */
  async removePersonFromGroup(groupId: string, personId: string) {
    const [relation] = await this.db
      .delete(schema.personToGroup)
      .where(eq(schema.personToGroup.personId, personId))
      .where(eq(schema.personToGroup.groupId, groupId))
      .returning();
    
    if (!relation) {
      throw new NotFoundException(`Person with ID ${personId} is not in group with ID ${groupId}`);
    }
    
    return relation;
  }

  /**
   * Get all persons in a group
   */
  async getPersonsInGroup(groupId: string) {
    // Check if the group exists
    await this.findById(groupId);
    
    // Get all persons in the group
    return this.db
      .select({
        person: schema.persons,
      })
      .from(schema.personToGroup)
      .innerJoin(schema.persons, eq(schema.personToGroup.personId, schema.persons.id))
      .where(eq(schema.personToGroup.groupId, groupId));
  }
}